from pydantic import BaseModel
from bs4 import BeautifulSoup


class Trending(BaseModel):
    preview: str
    title: str
    url: str

    views: str = 0


class Page(BaseModel):
    title: str
    published_at: str

    views: str
    comments: str

    author_name: str

    main_image: str
    text: str
    trending: list[Trending]

    @property
    def cleared_text(self) -> BeautifulSoup:
        soup = BeautifulSoup(self.text)

        for tag in [
            *soup.find_all("time"),
            *soup.find_all("img", class_=["avatar"]),
            *soup.find_all("span", class_=["count"]),
            *soup.find_all("span", itemprop="name"),
            *soup.find_all("div", class_=["mistape_caption", "share-box"]),
            *soup.find_all("h1"),
            *soup.find_all("hr"),
            *soup.find_all("figure", class_="bb-mb-el")
        ]:
            tag.replace_with("")

        for tag in soup.find_all("div", class_="su-quote-inner"):
            tag.name = "blockquote"

        for tag in soup.find_all("span", class_="su-quote-cite"):
            tag.name = "cite"

        for tag in soup.find_all("a"):
            if tag["href"] == "https://t.me/memepedia_Ru":
                tag.replace_with("")

        for tag in soup.find_all("a"):
            if tag["href"].startswith("https://memepedia.ru/"):
                tag["href"] = "/memepedia/" + tag["href"].removeprefix("https://memepedia.ru/")

        return soup



from bs4 import BeautifulSoup

from .models import Page, Trending
from .aioget import aioget


async def page(name: str) -> Page:
    page = await aioget(f"https://memepedia.ru/{name}/")

    soup = BeautifulSoup(page, "lxml")
    post = soup.find_all(class_="s-post-main")[0]

    try:
        image = post.find_all("figure", class_="post-thumbnail", recursive=True)[0].img["src"]
    except:
        image = post.find_all("div", class_="bb-media-placeholder")[0].img["src"]

    try:
        trending_ = soup.find_all(class_="widget_trending_entries")[0].ul
    except:
        trending_ = []

    try:
        comments = (
            soup.find_all("a", class_="post-comments")[0]
                .find_all("span", class_="count")[0].text
        )
    except:
        comments = 0

    return Page(
        title=post.h1.text,
        published_at=post.find_all("time", class_="published")[0].text,

        author_name=(
            soup.find_all("div", class_="author-info")[0]
                .find_all("a", class_="auth-url")[0]
                .span.text
        ),

        views=(
            soup.find_all("span", class_="post-views")[0]
                .find_all("span", class_="count")[0].text
        ),
        comments=comments,

        main_image=image,

        text=str(post),
        trending=[Trending(
            preview=trending.find_all("img")[0]["src"],
            title=trending.find_all(class_="content")[0].find_all("a")[0].text,
            url="/" + trending.find_all(class_="content")[0].find_all("a")[0]["href"].split("/")[-2]
        ) for trending in trending_]
    )




from bs4 import BeautifulSoup

from .models import Trending
from .aioget import aioget


async def popular() -> list[Trending]:
    page = await aioget(f"https://memepedia.ru/memoteka/")

    soup = BeautifulSoup(page, "lxml")

    return [Trending(
        preview=trending.find_all("img")[0]["src"],
        title=trending.find_all(class_="entry-title")[0].find_all("a")[0].text,
        url="/" + trending.find_all(class_="entry-title")[0].find_all("a")[0]["href"].split("/")[-2],
        views=(
            trending.find_all("span", class_="post-views")[0]
                    .find_all("span", class_="count")[0].text
        )
    ) for trending in [
        *soup.find_all(class_="post-item"),
    ]]

